@@ -574,7 +574,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 	E2kContext *ctx;
 	const char *modtime, *str, *prop;
 	char *uid;
-	char *tzid;
+	const char *tzid;
 	int status, i, priority, percent;
 	float f_percent;
 	ECalComponent *ecal, *ecomp;
@@ -641,7 +641,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 						   E2K_PR_DAV_LAST_MODIFIED);
 
 		e_cal_backend_exchange_cache_lock (cbex);
-		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
+		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href, NULL)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
 					     g_strdup (modtime));
@@ -651,8 +651,10 @@ get_changed_tasks (ECalBackendExchange *cbex)
 		e_cal_backend_exchange_add_timezone (cbex, icalcomp);
 
 		itt = icaltime_from_timet (e2k_parse_timestamp (modtime), 0);
-		if (!icaltime_is_null_time (itt))
+		if (!icaltime_is_null_time (itt)) {
+			e_cal_backend_exchange_ensure_utc_zone (E_CAL_BACKEND (cbex), &itt);
 			e_cal_component_set_last_modified (ecal, &itt);
+		}
 
 		/* Set Priority */
 		if ((str = e2k_properties_get_prop (result->props,
@@ -683,6 +685,8 @@ get_changed_tasks (ECalBackendExchange *cbex)
 				E2K_PR_HTTPMAIL_DATE))) {
 			itt = icaltime_from_timet (e2k_parse_timestamp (str), 0);
 			if (!icaltime_is_null_time (itt)) {
+				e_cal_backend_exchange_ensure_utc_zone (E_CAL_BACKEND (cbex), &itt);
+
 				e_cal_component_set_dtstamp (
 					E_CAL_COMPONENT (ecal), &itt);
 				e_cal_component_set_created (
@@ -790,8 +794,10 @@ get_changed_tasks (ECalBackendExchange *cbex)
 		if ((str = e2k_properties_get_prop (result->props,
 				E2K_PR_CALENDAR_LAST_MODIFIED))) {
 			itt = icaltime_from_timet (e2k_parse_timestamp(str), 0);
-			if (!icaltime_is_null_time (itt))
+			if (!icaltime_is_null_time (itt)) {
+				e_cal_backend_exchange_ensure_utc_zone (E_CAL_BACKEND (cbex), &itt);
 				e_cal_component_set_last_modified (ecal, &itt);
+			}
 		}
 
 		/* Set CATEGORIES */
@@ -831,7 +837,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 			e_cal_backend_exchange_cache_unlock (cbex);
 
 			if (status && kind == ICAL_VTODO_COMPONENT) {
-				char *str = icalcomponent_as_ical_string (icalcomp);
+				char *str = icalcomponent_as_ical_string_r (icalcomp);
 				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbex), str);
 				g_free (str);
 			}
@@ -1036,6 +1042,7 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	ECalComponent *comp;
 	icalcomponent *icalcomp, *real_icalcomp;
 	icalcomponent_kind kind;
+	icalproperty *icalprop;
 	struct icaltimetype current;
 	char *from_name = NULL, *from_addr = NULL;
 	char *boundary = NULL;
@@ -1069,9 +1076,18 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_InvalidObject;
         }
 
-	current = icaltime_from_timet (time (NULL), 0);
-	icalcomponent_add_property (icalcomp, icalproperty_new_created (current));
-	icalcomponent_add_property (icalcomp, icalproperty_new_lastmodified (current));
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_CREATED_PROPERTY);
+	if (icalprop)
+		icalproperty_set_created (icalprop, current);
+	else
+		icalcomponent_add_property (icalcomp, icalproperty_new_created (current));
+
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_LASTMODIFIED_PROPERTY);
+	if (icalprop)
+		icalproperty_set_lastmodified (icalprop, current);
+	else
+		icalcomponent_add_property (icalcomp, icalproperty_new_lastmodified (current));
 
 	modtime = e2k_timestamp_from_icaltime (current);
 
@@ -1085,7 +1101,7 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_backend_exchange_cache_lock (ecalbex);
 	/* check if the object is already present in our cache */
 	if (e_cal_backend_exchange_in_cache (E_CAL_BACKEND_EXCHANGE (backend),
-					     temp_comp_uid, modtime, NULL)) {
+					     temp_comp_uid, modtime, NULL, NULL)) {
 		e_cal_backend_exchange_cache_unlock (ecalbex);
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
@@ -1257,7 +1273,7 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
         e_cal_component_set_icalcomponent (new_comp, icalcomp);
 
 	/* Set the last modified time on the component */
-        current = icaltime_from_timet (time (NULL), 0);
+        current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
         e_cal_component_set_last_modified (new_comp, &current);
 
 	/* Set Attachments */
@@ -1330,14 +1346,14 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
 
 	for (l = comps; l; l = l->next) {
                 const char *uid;
-                char *obj, *rid = NULL;
+                char *calobj, *rid = NULL;
 
                 subcomp = l->data;
 
                 ecalcomp = e_cal_component_new ();
                 e_cal_component_set_icalcomponent (ecalcomp, subcomp);
 
-                current = icaltime_from_timet (time (NULL), 0);
+                current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
                 e_cal_component_set_created (ecalcomp, &current);
                 e_cal_component_set_last_modified (ecalcomp, &current);
 
@@ -1345,7 +1361,6 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
 
 		e_cal_component_get_uid (ecalcomp, &uid);
                 rid = e_cal_component_get_recurid_as_string (ecalcomp);
-		obj = (char *) e_cal_component_get_as_string (ecalcomp);
 
                 /*see if the object is there in the cache. if found, modify object, else create object*/
 
@@ -1354,27 +1369,28 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
                         char *old_object;
 
 			e_cal_backend_exchange_cache_unlock (cbex);
-                        status = modify_task_object (backend, cal, obj, CALOBJ_MOD_THIS, &old_object, NULL);
+                        status = modify_task_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL);
                         if (status != GNOME_Evolution_Calendar_Success) {
 				g_free (rid);
                                 goto error;
 			}
 
-                        e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, obj);
+                        e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, calobj);
 			g_free (old_object);
                 } else {
                         char *returned_uid;
 
 			e_cal_backend_exchange_cache_unlock (cbex);
-			status = create_task_object (backend, cal, &obj, &returned_uid);
+			calobj = (char *) icalcomponent_as_ical_string_r (subcomp);
+			status = create_task_object (backend, cal, &calobj, &returned_uid);
                         if (status != GNOME_Evolution_Calendar_Success) {
-				g_free (obj);
+				g_free (calobj);
 				g_free (rid);
                                 goto error;
 			}
 
-                        e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), obj);
-			g_free (obj);
+                        e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), calobj);
+			g_free (calobj);
                 }
 		g_free (rid);
         }
